# File for high level functions for Wanda

import pywanda
from wandalib import *

def get_transient_results(wanda_file: pywanda.WandaModel, pipes: list[str]):
    df_press = get_transient_pressures(wanda_file, pipes)
    graph_transient_pressures(df_press)
    df_head, profile = get_transient_heads(wanda_file, pipes)
    graph_transient_head(df_head, profile)
    return df_press, df_head, profile


from dotenv import load_dotenv
import os
import pywanda
from wandalib import get_transient_results
import matplotlib.pyplot as plt

load_dotenv()
cwd =  os.getenv('WANTA_TEST_DIR')
wanda_bin = os.getenv('WANDA_BIN') 

wanda_name = r'Testmodel.wdi'
wanda_file = os.path.join(cwd, wanda_name)
wanda_model = pywanda.WandaModel(wanda_file, wanda_bin)
wanda_model.reload_output()
print(get_transient_results(wanda_model, ['PIPE LINE MOMRAH 1']))
# print(get_transient_results(wanda_model, ['PIPE LINE START 1', 'PIPE LINE START 2', 'PIPE LINE START 3', 'PIPE LINE START 4']))
plt.show()
# print(dir(wandalib))



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from enum import Enum
import shutil
from dataclasses import dataclass
import pywanda
import os
import inspect

class AllowedProperties(Enum):
    ROUGHNESS = "Wall roughness"
    FLOW = "Initial delivery rate"

@dataclass
class Scenario:
    scenario_name: str
    parameters: dict
    
# Example of Scenario
#     scenario_name="CLOSURE_MOMRAH_VALVE", 
#     parameters={
#     "VALVE MOMRAH": {
#         "Action table": assign_closing_time(20)
#     }}
#     ), 
#                  Scenario(
#     scenario_name="CLOSURE_FARM_VALVE", 
#     parameters={
#     "VALVE FARM": {
#         "Action table": assign_closing_time(64)
#     }}
#     )
#     ]
# create_scenarios(wanda_file, transient_Scenario, wanda_bin, isUnsteady=True)

def get_all_components_in_model(wanda_model: pywanda.WandaModel) -> list[str]:
    return wanda_model.get_all_components_str()

def get_all_components_dict(wanda_model: pywanda.WandaModel) -> dict[str, list[str]]:
    all_components = get_all_components_in_model(wanda_model)
    components = {}
    for component in all_components:
        component_splited = component.split(" ", 1)
        type_ = component_splited[0]  # Avoid using 'type' as it's a built-in function
        name = component_splited[1]
        # Initialize the list if the key does not exist
        if type_ not in components:
            components[type_] = []
        components[type_].append(name)
    return components

def get_components_from_type(all_components: dict, type:str) -> list[str]:
    return all_components[type]

def show_components_from_type_str(filter: str, all_components: dict) -> str:
    list = get_components_from_type(all_components, filter)
    str = ""
    for component in list[filter]:
        str += filter + " " + component + "\n"
    return str

#################################
    
def create_wanda_model(wanda_file: str, wanda_bin: str) -> tuple[pywanda.WandaModel, str]:
    wanda_model = pywanda.WandaModel(wanda_file, wanda_bin)
    wanda_name = os.path.splitext(os.path.basename(wanda_file))[0]
    return wanda_model, wanda_name


def check_if_element_exist(component: str, all_elements: list) -> bool:
    splited_str = component.split()
    component_type = splited_str[0]
    component_name = ' '.join(splited_str[1:])
    if component_type in all_elements and component_name in all_elements[component_type]:
        return True
    else:
        return False
    
def create_scenarios(wanda_file: str, scenarios: list[Scenario], wanda_bin: str, isUnsteady: bool = False)-> None:
    if isUnsteady == True:
        results_dir = "transient_results"
    else:
        results_dir = "steady_results"
    cwd = os.path.dirname(wanda_file)
    try:
        os.mkdir(cwd + results_dir)
        print(f"Directory '{results_dir}' created successfully.")
    except FileExistsError:
        print(f"Directory '{results_dir}' already exists.")
    except PermissionError:
        print(f"Permission denied: Unable to create '{results_dir}'.")
    except Exception as e:
        print(f"An error occurred: {e}")
    mother_case_skeleton = os.path.join(cwd, os.path.splitext(os.path.basename(wanda_file))[0] + ".wdx")
    for scenario in scenarios:
        scenario_wdi = scenario.scenario_name + ".wdi"
        scenario_wdx = scenario.scenario_name + ".wdx"
        scenario_path = os.path.join(cwd, results_dir, scenario_wdi)
        print(wanda_file)
        print(scenario_path)
        scenario_skeleton_path = os.path.join(cwd, results_dir, scenario_wdx)
        print(mother_case_skeleton)
        print(scenario_skeleton_path)
        shutil.copy(wanda_file, scenario_path)
        shutil.copy(mother_case_skeleton, scenario_skeleton_path)
        new_wanda_model = pywanda.WandaModel(scenario_path, wanda_bin)
        for parameter in scenario.parameters:
            # TODO parameterType = parameter.split()[0]
            # if parameterType == "Signal":
            #     signal = new_wanda_model.get_signal_line(parameter)
            if parameter == "GLOBAL PROPERTY":
                for key, value in scenario.parameters[parameter].items():
                    property = new_wanda_model.get_property(key)
                    property.set_scalar(value)
            elif parameter == "SIGNAL DISUSE":
                for signal in scenario.parameters[parameter]:
                    signal_node = new_wanda_model.get_signal_line(f"Signal {signal}")
                    signal_node.set_disused(True)
            else:
                for key, value in scenario.parameters[parameter].items():
                    # print(get_all_elements(new_wanda_model))
                    component = new_wanda_model.get_component(parameter)
                    property = component.get_property(key)
                    if key == "Action table":
                        table = property.get_table()
                        table.set_float_data(value)
                        continue
                    property.set_scalar(value)
        print(f"Scenario %s created in path %s" % (scenario.scenario_name, cwd))        
        print("Running scenario...")
        if isUnsteady == True:
            new_wanda_model.run_unsteady()
        else:        
            new_wanda_model.run_steady()
        print("Scenario ran")        
        new_wanda_model.close()
        

def assign_closing_time(closing_time: int, offset_time: int = 10):
    time = [0, offset_time, closing_time + offset_time]
    position = [1, 1, 0]
    return [time, position]
    
def assing_value(wanda_model, component, parameter, value):
    component = wanda_model.get_component(component)
    flow_rate = component.get_property(parameter)
    flow_rate.set_scalar(value/3600)


import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from utils import AllowedProperties


def get_max_min_prv_pipes(wanda_model, elements):
    time_steps = wanda_model.get_time_steps()
    for el in elements:
        fig, ax = plt.subplots()
        fig, ax2 = plt.subplots()
        fig, bx = plt.subplots()
        component = wanda_model.get_component(el)
        pressure1 = np.array(component.get_property("Pressure 1").get_series()) / 100000
        pressure2 = np.array(component.get_property("Pressure 2").get_series()) / 100000
        discharge1 = np.array(component.get_property("Discharge 1").get_series()) * 3600
        P1 = pd.Series(pressure1, index=time_steps)
        P2 = pd.Series(pressure2, index=time_steps)
        D = pd.Series(discharge1, index=time_steps)
        print("For the element: ", component.get_name())
        print("For before the minimum pressure is: ", min(pressure1))
        print("For before the maximum pressure is: ", max(pressure1))
        print("For after the minimum pressure is: ", min(pressure2))
        print("For after the maximum pressure is: ", max(pressure2))
        print("For before the minimum flow is: ", min(discharge1))
        print("For before the maximum flow is: ", max(discharge1))
        ax.plot(P1, label="Pressure Before Pipe")
        ax2.plot(P2, label="Pressure After Pipe")
        ax.set_title(el)
        ax2.set_title(el)
        ax.grid()
        ax2.grid()
        ax.legend()
        ax2.legend()
        ax.set_xlabel("Time [s]")
        ax2.set_xlabel("Time [s]")
        ax.set_ylabel("Pressure [barg]")
        ax2.set_ylabel("Pressure [barg]")
        bx.plot(D)
        bx.set_title(el)
        bx.grid()
        bx.legend()
        bx.set_xlabel("Time [s]")
        bx.set_ylabel("Discharge [m3/hr]")
        bx.set(xlim=(0, time_steps[-1]))
        
        
def get_info_nodes(wanda_model, nodes, title=False):
    fig, bx = plt.subplots()
    time_steps = wanda_model.get_time_steps()
    for el in nodes:
        component = wanda_model.get_node(el)
        pressure = np.array(component.get_property("Pressure").get_series()) / 100000
        P1 = pd.Series(pressure, index=time_steps)
        print("For the element: ", component.get_name())
        print("The minimum pressure is: ", min(pressure))
        print("The maximum pressure is: ", max(pressure))
        bx.plot(P1, label="Pressure in Node")
        if title != False and isinstance(title, str) :
            bx.set_title(title)
        else:
            bx.set_title(el)
        bx.grid()
        bx.legend()
        bx.set_xlabel("Time [s]")
        bx.set_ylabel("Pressure [barg]")
        bx.set_xlim(left=0)
        
def get_node_pressure_series(wanda_model, node):    
    time_steps = wanda_model.get_time_steps()
    component = wanda_model.get_node(node)
    pressure = np.array(component.get_property("Pressure").get_series()) / 100000
    pressure_serie = pd.Series(pressure, index=time_steps)
    print("The minimum pressure for node ", component.get_name(), "is: ", min(pressure_serie))
    print("The maximum pressure for node ", component.get_name(), "is: ", max(pressure_serie))
    return pressure_serie

def get_node_pressure_transient(wanda_model, node):   
    node = wanda_model.get_node(node)
    wanda_model.read_node_output(node) #is necessary?
    time_steps = wanda_model.get_time_steps()
    node_pressure = np.array(node.get_property("Pressure").get_series()) / 100000
    node_pressure = pd.Series(node_pressure, index=time_steps)
    return node_pressure
        
    

def get_pressure_valves(wanda_model, valves):
    fig_num = 0
    fig, axs = plt.subplots(1, len(valves))
    fig.set_figwidth(10)
    fig.tight_layout()
    time_steps = wanda_model.get_time_steps()
    for valve in valves:
        fig, bx = plt.subplots()
        component = wanda_model.get_component(valve)
        pressure1 = np.array(component.get_property("Pressure 1").get_series()) / 100000
        pressure2 = np.array(component.get_property("Pressure 2").get_series()) / 100000
        discharge1 = np.array(component.get_property("Discharge 1").get_series()) * 3600
        P1 = pd.Series(pressure1, index=time_steps)
        P2 = pd.Series(pressure2, index=time_steps)
        D = pd.Series(discharge1, index=time_steps)
        print("For before the valve ", component.get_name(), "the minimum pressure is: ", min(pressure1))
        print("For before the valve ", component.get_name(), "the maximum pressure is: ", max(pressure1))
        print("For after the valve ", component.get_name(), "the minimum pressure is: ", min(pressure2))
        print("For after the valve ", component.get_name(), "the maximum pressure is: ", max(pressure2))
        print("The minimum flow is: ", min(discharge1))
        print("The maximum flow is: ", max(discharge1))
        axs[fig_num].plot(P1, label="Pressure Before Pipe")
        axs[fig_num].plot(P2, label="Pressure After Pipe")
        axs[fig_num].set_title(valve)
        axs[fig_num].grid()
        axs[fig_num].legend()
        axs[fig_num].set_xlabel("Time [s]")
        axs[fig_num].set_ylabel("Pressure [barg]")
        fig_num += 1
        bx.plot(D)
        bx.set_title(valve)
        bx.grid()
        bx.legend()
        bx.set_xlabel("Time [s]")
        bx.set_ylabel("Discharge [m3/hr]")
        bx.set(xlim=(0, time_steps[-1]))
def get_minimum_head(wanda_model, head_node, h1, h2, control_node, minimum_pressure):
    
    maxiter = h1  # Initial guess for head (maximum)
    miniter = h2 # Initial guess for head (minimum)

    cached_results = {}  # Store results of previous function calls

    def get_pressure_from_head(n):
        # Check if the result for n is already cached
        if n in cached_results:
            return cached_results[n], n

        component = wanda_model.get_component(f"BOUNDH {head_node}")
        head =  component.get_property("Head at t = 0 [s]")
        head.set_scalar(n)
        wanda_model.run_steady()
        node = wanda_model.get_node(f"H-node {control_node}")
        NODE_PRESSURE = node.get_property("Pressure").get_scalar_float() / 100000

        # Cache the result for future use
        cached_results[n] = NODE_PRESSURE
        print("The return is: ", NODE_PRESSURE, n)
        # print("With head:", head.get_scalar_float(), "the pressure is:", NODE_PRESSURE, "and the difference for min pressure:", NODE_PRESSURE - minimum_pressure)
        return NODE_PRESSURE, n

    tolerance = 0.001  # Tolerance for the root

    while True:
        press_headmax, headmax = get_pressure_from_head(maxiter)
        press_headmin, headmin = get_pressure_from_head(miniter)

        # Check if the function value at the maximum is close to the target
        if press_headmax > 0 and abs(press_headmax - minimum_pressure) < tolerance:
            node_pressure = press_headmax
            head_result = headmax
            break

        # Check if the function value at the minimum is close to the target
        if press_headmin > 0 and abs(press_headmin - minimum_pressure) < tolerance:
            node_pressure = press_headmin
            head_result = headmin
            break

        # Calculate the midpoint and its corresponding function value
        mean = (maxiter + miniter) / 2
        press_mean, headmean = get_pressure_from_head(mean)

        # Adjust the bounds based on the function value at the midpoint
        if press_mean < minimum_pressure:
            miniter = mean
        else:
            maxiter = mean
        print("maxiter", maxiter, "miniter", miniter)
        # Check if the difference between maxiter and miniter is within tolerance
        if maxiter - miniter < tolerance:
            node_pressure = press_mean
            head_result = headmean
            break
        
    print("Result for head = ", head_result, "Pressure in Bu Hasa: ", node_pressure)

# Ejemplo de uso
# get_minimum_head(wanda_model, "B3", 500, 50, "C", 20.1)

    
def change_parameter(wanda_model, elements, parameter: AllowedProperties, value, is_unsteady = False):
    """Docstring

    Currently only working for roughness

    Returns:
    int:Returning value

   """
   
    if parameter == AllowedProperties.ROUGHNESS:
        coef = 1 / 1000
        
    for element in elements:
        component = wanda_model.get_component(element)
        element_parameter = component.get_property(parameter.value)
        element_parameter.set_scalar(value * coef)
        print("Now the component ", component.get_name(), "has a ", parameter.value, " of ", round(element_parameter.get_scalar_float() * (1/coef), 3))
        
    if is_unsteady == True:
        print("Is running transient...")
        wanda_model.run_unsteady()
        print("Done")
    else:
        print("Is running steady...")
        wanda_model.run_steady()
        print("Done")
    print("Closing simulation")
    wanda_model.close()


import matplotlib.pyplot as plt
import pandas as pd
import numpy as np

# Transient
## Pipeline
### Pressure

def graph_transient_pressures(df: pd.DataFrame, title="Pipeline Pressure"):
    fig, bx = plt.subplots()
    bx.plot(df["Steady Pressure"], label="Steady Pressure", color="orange")
    bx.plot(df["Maximum Pressure"], label="Maximum Pressure", color="red", linestyle="dashdot")
    bx.plot(df["Minimum Pressure"], label="Minimum Pressure", color="blue", linestyle="dashdot")
    bx.set(xlim=(df.index[0], df.index[-1]))
    plt.title(title)
    plt.xlabel("Distance [m]")
    plt.ylabel("Pressure [barg]")
    plt.grid()
    plt.legend()
    plt.show()
    return [fig, bx]

### Head

def graph_transient_head(df: pd.DataFrame, profile, title="Pipeline Pressure"):
    fig, bx = plt.subplots()
    bx.plot(profile, label="Profile", color="green")
    bx.plot(df["Steady Head"], label="Steady Head", color="orange")
    bx.plot(df["Maximum Head"], label="Maximum Head", color="red", linestyle="dashdot")
    bx.plot(df["Minimum Head"], label="Minimum Head", color="blue", linestyle="dashdot")
    bx.set(xlim=(df.index[0], df.index[-1]))
    plt.title(title)
    plt.xlabel("Distance [m]")
    plt.ylabel("Head [m]")
    plt.grid()
    plt.legend()
    plt.show()
    return [fig, bx]

# Steady

## Pipeline
### Pressure
def graph_steady_pressure(steady_curve: pd.Series, title="Pipeline Pressure"):
    fig, bx = plt.subplots()
    bx.plot(steady_curve, label="Steady State Pressure", color="blue")
    bx.set(xlim=(steady_curve.index[0], steady_curve.index[-1]))

    # Add plot details
    plt.title(title)
    plt.xlabel("Distance [m]")
    plt.ylabel("Pressure [barg]")
    plt.grid()
    plt.legend()    
    # plt.show() 
    return [fig, bx]
    

### Head


def graph_steady_head(steady_curve: pd.Series, profile: pd.Series, title:str ="Pipeline Head"):
    fig, bx = plt.subplots()
    bx.plot(profile, label="Profile", color="green")
    bx.plot(steady_curve, label="Steady State Head", color="orange")
    bx.set(xlim=(profile.index[0], profile.index[-1]))

    # Add plot details
    plt.title(title)
    plt.xlabel("Distance [m]")
    plt.ylabel("Head [m]")
    plt.grid()
    plt.legend()    
    # plt.show() 
    return [fig, bx]

####

def add_air_valves(bx: plt.Axes, coordinates: list[(float, float)], color: str="red", size:int =50, label:str ="Air Valves") -> None:
    """
    Añade triángulos (marcadores) en las coordenadas especificadas en el gráfico.

    Parámetros:
        bx (matplotlib.axes.Axes): El eje del gráfico donde se añadirán los triángulos.
        coordinates (list of tuples): Lista de coordenadas (x, y) donde se colocarán los triángulos.
        color (str): Color de los triángulos (por defecto es "red").
        size (int): Tamaño de los triángulos (por defecto es 50).
    """
    i = 0
    for (x, y) in coordinates:
        if i == 0:
            bx.scatter(x, y, marker="v", color=color, s=size, zorder=5, label=label)
            i += 1
            continue
        bx.scatter(x, y, marker="v", color=color, s=size, zorder=5)  # zorder asegura que estén encima de las líneas
   


from setuptools import setup, find_packages

VERSION = '0.0.1' 
DESCRIPTION = 'A Python package with functions for WANDA'
LONG_DESCRIPTION = 'My libary of Python functions for the Water Hammer software WANDA'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="wandalib", 
        version=VERSION,
        author="Juan David Guerrero",
        author_email="<juanguerrero09mc@gmail.com>",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=["pywanda==4.7.0a2", "numpy", "matplotlib", "pandas"], # add any additional packages that 

        keywords=['python', 'first package'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Intended Audience :: Research",
            "Intended Audience :: Industry",
            "Programming Language :: Python :: 3",
            "Operating System :: Microsoft :: Windows",
        ]
)


import pywanda
import numpy as np
import pandas as pd

# Transient
## Node
def get_node_pressure_steady(wanda_model:pywanda.WandaModel, node: str) -> list[float]:   
    node = wanda_model.get_node(node)
    wanda_model.read_node_output(node)
    node_pressure = node.get_property("Pressure").get_scalar_float() / 100000
    return node_pressure

## Pipeline

### Head
def get_transient_heads(wanda_model: pywanda.WandaModel, pipes: list[str], downsampling_factor: int =1, is_relative: bool =False) -> tuple[pd.DataFrame, pd.Series]:
    """
    Calculate and plot the steady-state head profile for a pipeline system.

    Args:
        wanda_model: The WANDA model object.
        pipes (list): List of pipe names to analyze.
        downsampling_factor (int, optional): Factor to downsample the data. Defaults to 1.
        is_returning_series (bool, optional): If True, returns data as Pandas Series. Defaults to False.
        is_relative (bool, optional): If True, treats pipe distances as relative. Defaults to False.

    Returns:
        tuple: A tuple containing the results DataFrame and profile Series, or individual Series if is_returning_series is True.
    """
    # Initialize lists to store profile and head data
    profile_x_values = []
    profile_y_values = []
    head_steady_values = []
    head_max_values = []
    head_min_values = []
    length_steps = []
    last_x = 0 

    for pipe in pipes:
        component = wanda_model.get_component(pipe)
        profile_data = component.get_property("Profile").get_table().get_float_data()
        profile_x = np.array(profile_data[0])  # X-distance data
        profile_y = np.array(profile_data[1])  # Height data

        # Get steady-state head data
        head_pipe = component.get_property("Head")
        steady = np.array(head_pipe.get_series_pipe())


        if is_relative:
            # For the first pipe, initialize the arrays
            if pipe == pipes[0]:
                profile_x_values = profile_x
                profile_y_values = profile_y
            else:
                # Extend x and y arrays for subsequent pipes (skip the first point to avoid overlap)
                updated_distance = profile_x[1:] + last_x
                profile_x_values = np.concatenate([profile_x_values, updated_distance])
                profile_y_values = np.concatenate([profile_y_values, profile_y[1:]])
            steps = np.linspace(profile_x[0], profile_x[-1], len(steady)) + last_x
            last_x = profile_x_values[-1]  # Update the last x-distance
        else:
            profile_x_values.extend(profile_x)
            profile_y_values.extend(profile_y)
            steps = np.linspace(profile_x[0], profile_x[-1], len(steady))

        length_steps = np.concatenate([length_steps, steps])
        
        head_steady_values.append(steady[:, 0])
        head_max_values.append(np.array(head_pipe.get_extr_max_pipe()))
        head_min_values.append(np.array(head_pipe.get_extr_min_pipe()))

    # Convert lists to numpy arrays
    head_steady_values = np.concatenate(head_steady_values)
    head_max_values = np.concatenate(head_max_values)
    head_min_values = np.concatenate(head_min_values)
    profile_x_values = np.array(profile_x_values)

    # Downsample data if required
    if downsampling_factor != 1:
        head_steady_values = head_steady_values[::downsampling_factor]
        head_max_values = head_max_values[::downsampling_factor]
        head_min_values = head_min_values[::downsampling_factor]
        profile_x_values = profile_x_values[::downsampling_factor]
        length_steps = length_steps[::downsampling_factor]  # Downsample length_steps as well

    # Create profile Series
    profile = pd.Series(profile_y_values, index=profile_x_values)

    # Create results DataFrame
    results_dic = {
        'Steady Head': head_steady_values,
        'Maximum Head': head_max_values,
        'Minimum Head': head_min_values,
    }
    results_data = pd.DataFrame(results_dic, index=length_steps)
    results_data.index.name = 'Distance (m)'

    return results_data, profile

### Pressure

def get_transient_pressures(wanda_model: pywanda.WandaModel, pipes: list[str], downsampling_factor:int =1, print_messages:bool = True) -> pd.DataFrame:
    """
    Genera series de presión para tuberías en un modelo Wanda.

    Args:
        wanda_model: Modelo Wanda que contiene los componentes de las tuberías.
        pipes (list): Lista de nombres de tuberías a procesar.
        downsampling_factor (int): Factor de muestreo para reducir el tamaño de las series. 
                                   El valor por defecto es 1 (sin reducción).
        print_messages (Bool): Printea los valores máximos y mínimos de presión en la tubería en el código
        is_returning_serie (Bool): Devuelve los valores de presion tres series de pandas en lugar de dataframe

    Returns:
        tuple: Series de presión estacionaria, mínima y máxima, como pandas.Series.
    """
    # time_steps = wanda_model.get_time_steps()
    length_steps = []
    steady_pressures = []
    max_pressures = []
    min_pressures = []
    for pipe in pipes:
        
        component = wanda_model.get_component(pipe)
        pressure_data = component.get_property("Pressure")
        profile_data = component.get_property("Profile").get_table().get_float_column("X-distance")

        pressure_series = np.array(pressure_data.get_series_pipe()) /100000
        steady_pressures.append(pressure_series[:, 0])
        
        max_pressures.append(np.array(pressure_data.get_extr_max_pipe()) / 100000)
        min_pressures.append(np.array(pressure_data.get_extr_min_pipe()) / 100000)
        
        length_steps.append(np.linspace(profile_data[0], profile_data[-1], len(pressure_series)))
        
        min_pressure = min(pressure_data.get_extr_min_pipe()) / 100000
        max_pressure = max(pressure_data.get_extr_max_pipe()) / 100000
        if print_messages:
            print(f"For pipeline '{component.get_name()}', min pressure: {min_pressure} bar, max pressure: {max_pressure} bar")

    # Convert lists to numpy arrays
    steady_pressures = np.concatenate(steady_pressures)
    max_pressures = np.concatenate(max_pressures)
    min_pressures = np.concatenate(min_pressures)
    length_steps = np.concatenate(length_steps)
    
    if downsampling_factor > 1:
        steady_pressures = steady_pressures[::downsampling_factor]
        max_pressures = max_pressures[::downsampling_factor]
        min_pressures = min_pressures[::downsampling_factor]
        length_steps = length_steps[::downsampling_factor]
    
    results_dic = {
        'Steady Pressure': steady_pressures,
        'Maximum Pressure': max_pressures,
        'Minimum Pressure': min_pressures,
    }
    
    results_data = pd.DataFrame(results_dic, index=length_steps)
    results_data.index.name = 'Distance (m)'
    
    return results_data

## Surge Vessel

def get_surge_vessel_serie(wanda_model, sv):
    time_steps = wanda_model.get_time_steps()
    component = wanda_model.get_component(sv)
    liquid_vol = np.array(component.get_property("Liquid volume").get_series())
    liquid_vol_serie = pd.Series(liquid_vol, index=time_steps)
    print("The minimum volume for the surge vessel ", component.get_name(), "is: ", min(liquid_vol_serie))
    print("The maximum volume for the surge vessel ", component.get_name(), "is: ", max(liquid_vol_serie))
    return liquid_vol_serie

############################################

# Steady

## Pipeline

### Head

def get_head_steady(wanda_model: pywanda.WandaModel, pipes:list[str], downsampling_factor:int =1, is_relative:bool =False):
    """
    Calculate and plot the steady-state head profile for a pipeline system.

    Args:
        wanda_model: The WANDA model object.
        pipes (list): List of pipe names to analyze.
        downsampling_factor (int, optional): Factor to downsample the data. Defaults to 1.
        is_relative (bool, optional): If True, treats pipe distances as relative. Defaults to False.

    Returns:
        list: A list containing the matplotlib figure and axis objects for the plot.
    """
    # Initialize lists to store profile and head data
    profile_x_values = []
    profile_y_values = []
    head_steady_values = []
    length_steps = []
    last_x = 0 
    
    
    for pipe in pipes:
        component = wanda_model.get_component(pipe)
        profile_data = component.get_property("Profile").get_table().get_float_data()
        profile_x = np.array(profile_data[0])  # X-distance data
        profile_y = np.array(profile_data[1])  # Height data
        
        # Get steady-state head data
        pressure_pipe = component.get_property("Head")
        series_pipe = np.array(pressure_pipe.get_series_pipe())
        steady = series_pipe[:, 0]
        
        if is_relative:
            # For the first pipe, initialize the arrays
            if pipe == pipes[0]:
                profile_x_values = profile_x
                profile_y_values = profile_y
            else:
                # Extend x and y arrays for subsequent pipes (skip the first point to avoid overlap)
                updated_distance = profile_x[1:] + last_x
                profile_x_values = np.concatenate([profile_x_values, updated_distance])
                profile_y_values = np.concatenate([profile_y_values, profile_y[1:]])
            steps = np.linspace(profile_x[0], profile_x[-1], len(steady)) + last_x
            last_x = profile_x_values[-1]  # Update the last x-distance
        else:
            profile_x_values.extend(profile_x)
            profile_y_values.extend(profile_y)
            steps = np.linspace(profile_x[0], profile_x[-1], len(steady))

        length_steps = np.concatenate([length_steps, steps])
        head_steady_values.append(steady)

    # Convert lists to numpy arrays
    head_steady_values = np.concatenate(head_steady_values)
    profile_x_values = np.array(profile_x_values)

    # Downsample data if required
    if downsampling_factor != 1:
        head_steady_values = head_steady_values[::downsampling_factor]
        profile_x_values = profile_x_values[::downsampling_factor]

    # Create pandas Series for plotting
    steady_curve = pd.Series(head_steady_values, index=length_steps)
    profile = pd.Series(profile_y_values, index=profile_x_values)
    
    # Plot the profile and steady-state head

    return steady_curve, profile

### Pressure
 
def get_pressure_steady(wanda_model: pywanda.WandaModel, pipes: list[str], downsampling_factor:int =1, is_relative:bool =False, show_messages: bool = False) -> pd.Series:
    """
    Calculate and plot the steady-state pressure profile for a pipeline system.

    Args:
        wanda_model: The WANDA model object.
        pipes (list): List of pipe names to analyze.
        downsampling_factor (int, optional): Factor to downsample the data. Defaults to 1.
        is_relative (bool, optional): If True, treats pipe distances as relative. Defaults to False.

    Returns:
        list: A list containing the matplotlib figure and axis objects for the plot.
    """
    # Initialize lists to store profile and pressure data
    pressure_steady_values = []
    len_steps = []
    last_x = 0 
    
    
    for pipe in pipes:
        component = wanda_model.get_component(pipe)
        
        # Get steady-state head data
  
        pressure_pipe = component.get_property("Pressure")
        series_pipe = np.array(pressure_pipe.get_series_pipe())/100000
        steady = series_pipe[:, 0]
        profile_x = component.get_property("Profile").get_table().get_float_column("X-distance")
        profile_x_values = []
        
        if is_relative:
            # For the first pipe, initialize the arrays
            len_steps.append(np.linspace(profile_x[0], profile_x[-1], len(series_pipe)))
            if pipe == pipes[0]:
                profile_x_values = profile_x
            else:
                # Extend x and y arrays for subsequent pipes (skip the first point to avoid overlap)
                updated_distance = profile_x[1:] + last_x
                profile_x_values = np.concatenate([profile_x_values, updated_distance])
            steps = np.linspace(profile_x[0], profile_x[-1], len(steady)) + last_x
            last_x = profile_x_values[-1]  # Update the last x-distance
        else:
            profile_x_values.extend(profile_x)
            steps = np.linspace(profile_x[0], profile_x[-1], len(steady))

        len_steps = np.concatenate([len_steps, steps])
        pressure_steady_values.append(steady)

    # Convert lists to numpy arrays
    pressure_steady_values = np.concatenate(pressure_steady_values)
    profile_x_values = np.array(profile_x_values)

    # Downsample data if required
    if downsampling_factor != 1:
        pressure_steady_values = pressure_steady_values[::downsampling_factor]
        profile_x_values = profile_x_values[::downsampling_factor]

    # Create pandas Series for plotting
    steady_curve = pd.Series(pressure_steady_values, index=len_steps)
    
    if show_messages == True:
        print("For pipeline ", component.get_name(), "the minimum pressure is: ", min(steady_curve))
        print("For pipeline ", component.get_name(), "the maximum pressure is: ", max(steady_curve))
    # Plot the profile and steady-state pressure

    return steady_curve



